// $Id: kclock.cpp 622692 2007-01-12 16:51:49Z mlaurent $
//
// kclock - Clock screen saver for KDE
// Copyright (c) 2003, 2006   Melchior FRANZ
//
// License:		GPL v2
// Author:		Melchior FRANZ  [mfranz#kde:org]
//
#include <stdlib.h>
#include <string.h>

#include <QCheckBox>
#include <QColor>
#include <QDateTime>
#include <QGridLayout>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QImage>
#include <QLabel>
#include <QLayout>
#include <QPainter>
#include <QSlider>
#include <QVBoxLayout>

#include <khbox.h>
#include <kapplication.h>
#include <kcolorbutton.h>
#include <kconfig.h>
#include <kglobal.h>
#include <klocale.h>
#include <kmessagebox.h>

#include "kclock.h"
#include "kclock.moc"



const int COLOR_BUTTON_WIDTH = 80;
const int TIMER_INTERVAL = 100;
const int MAX_CLOCK_SIZE = 10;
const unsigned int DEFAULT_CLOCK_SIZE = 3;
const bool DEFAULT_KEEP_CENTERED = false;




class KClockSaverInterface : public KScreenSaverInterface {
public:
	virtual KAboutData* aboutData() {
		return new KAboutData("kclock.kss", I18N_NOOP("Clock"), "1.90", I18N_NOOP("Clock"));
	}

	virtual KScreenSaver* create(WId id) {
		return new KClockSaver(id);
	}

	virtual QDialog* setup() {
		return new KClockSetup();
	}
};


int main(int argc, char *argv[])
{
	KClockSaverInterface kss;
	return kScreenSaverMain(argc, argv, kss);
}




//-----------------------------------------------------------------------------

KClockSetup::KClockSetup(QWidget *parent) :
	KDialog(parent),
	_saver(0)
{
	setCaption(i18n("Setup Clock Screen Saver"));
	setModal(true);
	setButtons(Ok|Cancel|Help);
	setDefaultButton(Ok);

	readSettings();

	setButtonText(Help, i18n("A&bout"));
	QWidget *main = new QWidget(this);
	setMainWidget(main);

	QVBoxLayout *top = new QVBoxLayout(main);

	QHBoxLayout *hbox = new QHBoxLayout;
	top->addLayout(hbox);


	QGroupBox *colgroup = new QGroupBox(i18n("Colors"), main);
	QGridLayout *grid = new QGridLayout();

	QLabel *label;
	KColorButton *colorButton;

	label = new QLabel(i18n("&Hour-hand:"));
	colorButton = new KColorButton(_hourColor);
	colorButton->setFixedWidth(COLOR_BUTTON_WIDTH);
	label->setBuddy(colorButton);
	connect(colorButton, SIGNAL(changed(const QColor &)),
			SLOT(slotHourColor(const QColor &)));
	grid->addWidget(label, 1, 1);
	grid->addWidget(colorButton, 1, 2);

	label = new QLabel(i18n("&Minute-hand:"));
	colorButton = new KColorButton(_minColor);
	colorButton->setFixedWidth(COLOR_BUTTON_WIDTH);
	label->setBuddy(colorButton);
	connect(colorButton, SIGNAL(changed(const QColor &)),
			SLOT(slotMinColor(const QColor &)));
	grid->addWidget(label, 2, 1);
	grid->addWidget(colorButton, 2, 2);

	label = new QLabel(i18n("&Second-hand:"));
	colorButton = new KColorButton(_secColor);
	colorButton->setFixedWidth(COLOR_BUTTON_WIDTH);
	label->setBuddy(colorButton);
	connect(colorButton, SIGNAL(changed(const QColor &)),
			SLOT(slotSecColor(const QColor &)));
	grid->addWidget(label, 3, 1);
	grid->addWidget(colorButton, 3, 2);

	label = new QLabel(i18n("Scal&e:"));
	colorButton = new KColorButton(_scaleColor);
	colorButton->setFixedWidth(COLOR_BUTTON_WIDTH);
	label->setBuddy(colorButton);
	connect(colorButton, SIGNAL(changed(const QColor &)),
			SLOT(slotScaleColor(const QColor &)));
	grid->addWidget(label, 4, 1);
	grid->addWidget(colorButton, 4, 2);

	label = new QLabel(i18n("&Background:"));
	colorButton = new KColorButton(_bgndColor);
	colorButton->setFixedWidth(COLOR_BUTTON_WIDTH);
	label->setBuddy(colorButton);
	connect(colorButton, SIGNAL(changed(const QColor &)),
			SLOT(slotBgndColor(const QColor &)));
	grid->addWidget(label, 5, 1);
	grid->addWidget(colorButton, 5, 2);

	hbox->addWidget(colgroup);
	colgroup->setLayout(grid);


	QWidget *_preview = new QWidget(main);
	_preview->setFixedSize(220, 165);
	_preview->show();
	_saver = new KClockSaver(_preview->winId());
	hbox->addWidget(_preview);

	label = new QLabel(i18n("Si&ze:"), main);
	top->addWidget(label);
	QSlider *qs = new QSlider(Qt::Horizontal);
	label->setBuddy(qs);
	qs->setRange(0, MAX_CLOCK_SIZE);
	qs->setSliderPosition(_size);
	qs->setTickInterval(1);
	qs->setTickPosition(QSlider::TicksBelow);
	connect(qs, SIGNAL(valueChanged(int)), this, SLOT(slotSliderMoved(int)));
	top->addWidget(qs);

	bool rtl = kapp->layoutDirection();
	KHBox *qsscale = new KHBox(main);
	label = new QLabel(i18n("Small"), qsscale);
	label->setAlignment(rtl ? Qt::AlignRight : Qt::AlignLeft);
	label = new QLabel(i18n("Medium"), qsscale);
	label->setAlignment(Qt::AlignHCenter);
	label = new QLabel(i18n("Big"), qsscale);
	label->setAlignment(rtl ? Qt::AlignLeft : Qt::AlignRight);
	top->addWidget(qsscale);

	QCheckBox *keepCentered = new QCheckBox(i18n("&Keep clock centered"), main);
	keepCentered->setChecked(_keepCentered);
	connect(keepCentered, SIGNAL(stateChanged(int)), SLOT(slotKeepCenteredChanged(int)));
	top->addWidget(keepCentered);
	top->addStretch();
	connect(this,SIGNAL(okClicked()),this,SLOT(slotOk()));
	connect(this,SIGNAL(helpClicked()),this,SLOT(slotHelp()));
}


KClockSetup::~KClockSetup()
{
	delete _saver;
}


void KClockSetup::readSettings()
{
	KConfig *config = KGlobal::config();

	config->setGroup("Settings");
	_keepCentered = config->readEntry("KeepCentered", DEFAULT_KEEP_CENTERED);
	_size = config->readEntry("Size", DEFAULT_CLOCK_SIZE);
	if (_size > MAX_CLOCK_SIZE)
		_size = MAX_CLOCK_SIZE;

	config->setGroup("Colors");
	QColor c = Qt::black;
	_bgndColor = config->readEntry("Background", c);

	c = Qt::white;
	_scaleColor = config->readEntry("Scale", c);
	_hourColor = config->readEntry("HourHand", c);
	_minColor = config->readEntry("MinuteHand", c);

	c = Qt::red;
	_secColor = config->readEntry("SecondHand", c);

	if (_saver) {
		_saver->setBgndColor(_bgndColor);
		_saver->setScaleColor(_scaleColor);
		_saver->setHourColor(_hourColor);
		_saver->setMinColor(_minColor);
		_saver->setSecColor(_secColor);
	}
}


void KClockSetup::slotOk()
{
	KConfig *config = KGlobal::config();
	config->setGroup("Settings");
	config->writeEntry("Size", _size);
	config->writeEntry("KeepCentered", _keepCentered);

	config->setGroup("Colors");
	config->writeEntry("Background", _bgndColor);
	config->writeEntry("Scale", _scaleColor);
	config->writeEntry("HourHand", _hourColor);
	config->writeEntry("MinuteHand", _minColor);
	config->writeEntry("SecondHand", _secColor);
	config->sync();
	accept();
}


void KClockSetup::slotHelp()
{
	KMessageBox::about(this, "<qt>" + i18n(
			"Clock Screen Saver<br>"
			"Version 1.90<br>"
			"<nobr>Melchior FRANZ (c) 2003, 2006</nobr>") +
			"<br><a href=\"mailto:mfranz@kde.org\">mfranz@kde.org</a>"
			"</qt>", QString::null, KMessageBox::AllowLink);
}


void KClockSetup::slotBgndColor(const QColor &color)
{
	_bgndColor = color;
	if (_saver)
		_saver->setBgndColor(_bgndColor);
}


void KClockSetup::slotScaleColor(const QColor &color)
{
	_scaleColor = color;
	if (_saver)
		_saver->setScaleColor(_scaleColor);
}


void KClockSetup::slotHourColor(const QColor &color)
{
	_hourColor = color;
	if (_saver)
		_saver->setHourColor(_hourColor);
}


void KClockSetup::slotMinColor(const QColor &color)
{
	_minColor = color;
	if (_saver)
		_saver->setMinColor(_minColor);
}


void KClockSetup::slotSecColor(const QColor &color)
{
	_secColor = color;
	if (_saver)
		_saver->setSecColor(_secColor);
}


void KClockSetup::slotSliderMoved(int v)
{
	if (_saver)
		_saver->restart(_size = v);
}


void KClockSetup::slotKeepCenteredChanged(int c)
{
	if (_saver)
		_saver->setKeepCentered(_keepCentered = c);
}





//-----------------------------------------------------------------------------

ClockPainter::ClockPainter(QWidget *parent) :
	QPainter(parent)
{
	setRenderHint(QPainter::Antialiasing, true);
}


void ClockPainter::drawTick(double angle, double from, double to, double width, QColor color, bool shadow)
{
	save();
	rotate(90.0 - angle);
	setPen(Qt::NoPen);

	if (shadow) {
		width += 1.0;
		setBrush(QColor(100, 100, 100));
	} else {
		setBrush(color);
	}
	drawRect(QRectF(from, -width / 2.0, to - from, width));
	restore();
}


void ClockPainter::drawDisc(double width, QColor color, bool shadow)
{
	setPen(Qt::NoPen);

	if (shadow) {
		width += 1.0;
		setBrush(QColor(100, 100, 100));
	} else {
		setBrush(color);
	}
	drawEllipse(QRectF(-width, -width, 2.0 * width, 2.0 * width));
}


void ClockPainter::drawHand(double angle, double length, double width, QColor color, bool disc)
{
	if (disc)
		drawDisc(width * 1.3, color, true);
	drawTick(angle, 0.0, length, width, color, true);

	if (disc)
		drawDisc(width * 1.3, color, false);
	drawTick(angle, 0.0, length, width, color, false);

}


void ClockPainter::drawScale(QColor color)
{
	for (int i = 0; i < 360; i += 6)
		if (i % 30)
			drawTick(i, 920.0, 980.0, 15.0, color);
		else
			drawTick(i, 825.0, 980.0, 40.0, color);
}



//====================================================================================



ClockWidget::ClockWidget(QWidget *parent) :
	QWidget(parent),
	_hour(-1),
	_minute(-1),
	_second(-1),
	_showSecond(true)
{
	_timer = new QTimer(this);
	connect(_timer, SIGNAL(timeout()), this, SLOT(timerSlot()));
	_timer->start(100);
}


ClockWidget::~ClockWidget()
{
	delete _timer;
}


void ClockWidget::timerSlot(void)
{
	QTime t = QTime::currentTime();
	int s = t.second();
	if (s == _second)
		return;

	_second = _showSecond ? s : 0;
	_second = s;
	_hour = t.hour();
	_minute = t.minute();
	update();
}


void ClockWidget::paintEvent(QPaintEvent *)
{
	double hour_angle = _hour * 30.0 + _minute * .5;
	double minute_angle = _minute * 6.0 + _second * .1;
	double second_angle = _second * 6.0;

	ClockPainter clock(this);
	clock.fillRect(0, 0, 2 * width(), 2 * height(), QBrush(Qt::black));
	clock.translate(width() / 2, height() / 2);
	clock.scale(0.8 * qMin(width(), height()));

	clock.drawScale(Qt::white);
	clock.drawHand(hour_angle, 600.0, 55.0, Qt::white, false);
	clock.drawHand(minute_angle, 900.0, 40.0, Qt::white);
	clock.drawHand(second_angle, 900.0, 30.0, Qt::red);
}



//-----------------------------------------------------------------------------

KClockSaver::KClockSaver(WId id) :
	KScreenSaver(id),
	_showSecond(true)
{
	readSettings();

	_clock_widget = new ClockWidget(this);
	_clock_widget->resize(width(), height());
	_clock_widget->show();
}


KClockSaver::~KClockSaver()
{
	delete _clock_widget;
}


void KClockSaver::start(int /*size*/)
{
	//_diameter = int(QMIN(width(), height()) * (size + 4) / 14.0);
	//_x = (width() - _diameter) / 2;
	//_y = (height() - _diameter) / 2;

	//_image = new QImage(_diameter, _diameter, 32);
	//_scale = new KClockPainter(_diameter, _diameter);
	//_clock = new ClockPainter();//_diameter, _diameter);

	//_clock->setShadowColor(qRgb((_bgndColor.red() + _scaleColor.red()) / 2,
	//		(_bgndColor.green() + _scaleColor.green()) / 2,
	//		(_bgndColor.blue() + _scaleColor.blue()) / 2));
	//drawScale();
	//forceRedraw();
}


void KClockSaver::stop()
{
	//delete _clock;
	//delete _scale;
	//delete _image;
}


void KClockSaver::restart(int /*size*/)
{
	//_timer.stop();
	//stop();
	//start(size);
	//drawScale();
	//_timer.start(TIMER_INTERVAL);
}


void KClockSaver::setKeepCentered(bool /*b*/)
{
	//_keepCentered = b;
	//if (b) {
	//	_x = (width() - _diameter) / 2;
	//	_y = (height() - _diameter) / 2;
	//}
	//forceRedraw();
}


void KClockSaver::readSettings()
{
	KConfig *config = KGlobal::config();

	config->setGroup("Settings");
	_keepCentered = config->readEntry("KeepCentered", DEFAULT_KEEP_CENTERED);
	_size = config->readEntry("Size", DEFAULT_CLOCK_SIZE);
	if (_size > MAX_CLOCK_SIZE)
		_size = MAX_CLOCK_SIZE;

	config->setGroup("Colors");
	QColor c = Qt::black;
	_bgndColor = config->readEntry("Background", c);

	c = Qt::white;
	_scaleColor = config->readEntry("Scale", c);
	_hourColor = config->readEntry("HourHand", c);
	_minColor = config->readEntry("MinuteHand", c);

	c = Qt::red;
	_secColor = config->readEntry("SecondHand", c);
}


void KClockSaver::drawScale()
{
	//_scale->fill(_bgndColor);
	//_scale->setColor(_scaleColor);

	//for (int i = 0; i < 360; i += 6)
	//	if (i % 30)
	//		_scale->drawRadial(i, 920, 980, 15);
	//	else
	//		_scale->drawRadial(i, 825, 980, 40);

	//forceRedraw();
}


void KClockSaver::drawClock()
{
	//double hour_angle = _hour * 30.0 + _minute * .5;
	//double minute_angle = _minute * 6.0 + _second * .1;
	//double second_angle = _second * 6.0;

	//_clock->copy(_scale);
	//_clock->drawHand(hour_angle, 600, 55, _hourColor, false);
	//_clock->drawHand(minute_angle, 900, 40, _minColor);

	//if (_showSecond)
	//	_clock->drawHand(second_angle, 900, 30, _secColor);
}


void KClockSaver::slotTimeout()
{
	_clock_widget->update();
/*
	QTime t = QTime::currentTime();
	int s = t.second();
	if (s == _second)
		return;

	_second = _showSecond ? s : 0;
	_hour = t.hour();
	_minute = t.minute();

	drawClock();
	QPainter p(this);

	if (width() < 256) {
		// intended for the control module preview: always fill the whole area
		QImage *img = new QImage(width(), height(), 32);
		img->fill(qRgb(_bgndColor.red(), _bgndColor.green(), _bgndColor.blue()));
		//_clock->drawToImage(img, _x, _y);
		p.drawImage(0, 0, *img);
		delete img;
	} else {
		//_clock->drawToImage(_image);
		p.drawImage(_x, _y, *_image);
	}

	if (!_keepCentered) {
		static int xstep = 1;
		static int ystep = -1;
		int i;

		_x += xstep;
		if (_x <= 0)
			_x = 0, xstep = 1;
		else if (_x >= (i = width() - _diameter))
			_x = i, xstep = -1;

		_y += ystep;
		if (_y <= 0)
			_y = 0, ystep = 1;
		else if (_y >= (i = height() - _diameter))
			_y = i, ystep = -1;
	}
*/
}


