// $Id: kclock.h 563459 2006-07-17 17:03:47Z mfranz $
//
// kclock - Clock screen saver for KDE
// Copyright (c) 2003, 2006   Melchior FRANZ
//
// License:		GPL v2
// Author:		Melchior FRANZ  <mfranz@kde.org>
//
#ifndef __KCLOCK_H__
#define __KCLOCK_H__

#include <QTimer>
#include <QPainter>
#include <QWidget>

#include <kdialog.h>
#include <kscreensaver.h>

#include <iostream>				// FIXME
using namespace std;


class ClockPainter : public QPainter
{
public:
	ClockPainter(QWidget *parent = 0);
	virtual ~ClockPainter() {}
	void drawScale(QColor);
	void drawTick(double angle, double from, double to, double width, QColor, bool shadow = false);
	void drawDisc(double width, QColor, bool shadow = false);
	void drawHand(double angle, double length, double width, QColor, bool disc = true);
	void scale(qreal s) { s /= 2000.0; QPainter::scale(s, -s); }
};


class ClockWidget : public QWidget
{
	Q_OBJECT

public:
	ClockWidget(QWidget *parent = 0);
	~ClockWidget();

protected slots:
	void timerSlot();

protected:
	void paintEvent(QPaintEvent *event);

private:
	QTimer *_timer;
	int _hour;
	int _minute;
	int _second;
	bool _showSecond;
};


/*
class KClockPainter
{
	int _width;
	int _height;
	Q_UINT8 *_buf;
	double _matrix[6];
	Q_UINT32 _color;
	Q_UINT32 _shadow;

    public:
	KClockPainter(int width, int height);
	~KClockPainter();
	void copy(KClockPainter *p);
	void drawToImage(QImage *q, int x, int y);
	inline int width() { return _width; }
	inline int height() { return _height; }
	inline void *image() { return (void *)_buf; }
	void setColor(const QColor &color);
	void setShadowColor(const QColor &color);
	void fill(const QColor &color);
	void drawRadial(double alpha, double r0, double r1, double width);
	void drawDisc(double radius);
	void drawHand(const QColor &color, double angle, double length,
			double width, bool disc);
};
*/


class KClockSaver : public KScreenSaver
{
	Q_OBJECT

public:
	KClockSaver(WId id);
	virtual ~KClockSaver();
	inline void setBgndColor(const QColor &c) { _bgndColor = c; drawScale(); }
	inline void setScaleColor(const QColor &c) { _scaleColor = c; drawScale(); }
	inline void setHourColor(const QColor &c) { _hourColor = c; forceRedraw(); }
	inline void setMinColor(const QColor &c) { _minColor = c; forceRedraw(); }
	inline void setSecColor(const QColor &c) { _secColor = c; forceRedraw(); }
	void setKeepCentered(bool b);
	void restart(int siz);
	inline void forceRedraw() { _second = -1; }

private:
	void readSettings();
	void drawScale();
	void drawClock();
	void start(int size);
	void stop();

protected slots:
	void slotTimeout();

protected:
	ClockWidget *_clock_widget;

	QTimer _timer;
	QImage *_image;
	//KClockPainter *_scale;
	ClockPainter *_clock;

	int _x;
	int _y;
	int _diameter;
	int _size;
	bool _showSecond;
	bool _keepCentered;
	int _hour;
	int _minute;
	int _second;

	QColor _bgndColor;
	QColor _scaleColor;
	QColor _hourColor;
	QColor _minColor;
	QColor _secColor;
};


class KClockSetup : public KDialog
{
	Q_OBJECT
public:
	KClockSetup(QWidget *parent = 0);
	~KClockSetup();

protected:
	void readSettings();

private slots:
	void slotOk();
	void slotHelp();

	void slotBgndColor(const QColor &);
	void slotScaleColor(const QColor &);
	void slotHourColor(const QColor &);
	void slotMinColor(const QColor &);
	void slotSecColor(const QColor &);
	void slotSliderMoved(int);
	void slotKeepCenteredChanged(int);

private:
	KClockSaver *_saver;

	QColor _bgndColor;
	QColor _scaleColor;
	QColor _hourColor;
	QColor _minColor;
	QColor _secColor;

	int _size;
	bool _keepCentered;
};

#endif


